
CREATE OR REPLACE TYPE tp_endereco AS OBJECT(
Logradouro VARCHAR2(80),
Bairro VARCHAR2(40),
Cidade VARCHAR2(40),
CEP NUMBER(8,0),
Numero NUMBER(5,0),
Complemento NUMBER(4,0)
);
/

CREATE OR REPLACE TYPE tp_telefones AS VARRAY(2) OF NUMBER(10);
/

CREATE OR REPLACE TYPE tp_pessoa AS OBJECT(
IdPessoa NUMBER(5,0),
CPF NUMBER(11,0),
RG  NUMBER(7,0),
Nome VARCHAR2(80),
Sexo CHAR(1),
DataNascimento DATE,
Endereco tp_endereco,
Telefones tp_telefones
)NOT FINAL;
/

CREATE OR REPLACE TYPE tp_especialidade AS OBJECT(
Especialidade VARCHAR2(40)
);
/

CREATE OR REPLACE TYPE tp_listaespecialidade AS TABLE OF tp_especialidade;
/

CREATE OR REPLACE TYPE tp_medico UNDER tp_pessoa(
CRM NUMBER(6,0),
Especialidades tp_listaespecialidade
);
/

CREATE TABLE tb_medico OF tp_medico(
IdPessoa PRIMARY KEY
)NESTED TABLE Especialidades STORE AS Especialidade_ST;

CREATE OR REPLACE TYPE tp_cliente UNDER tp_pessoa(
CodPlano NUMBER(5,0),
EstadoCivil VARCHAR2(20),
TipoSanguineo VARCHAR2(3)
)NOT FINAL;
/

CREATE TABLE tb_cliente OF tp_cliente(
IdPessoa PRIMARY KEY
);

CREATE OR REPLACE TYPE tp_titular UNDER tp_cliente(
ValorPlano NUMBER(6,2)
);
/

CREATE OR REPLACE TYPE tp_dependente UNDER tp_cliente(
Parentesco VARCHAR2(20),
ref_titular REF tp_titular
);
/

CREATE TABLE tb_titular OF tp_titular(
IdPessoa PRIMARY KEY
);

CREATE TABLE tb_dependente OF tp_dependente(
IdPessoa PRIMARY KEY,
ref_titular WITH ROWID REFERENCES tb_titular
);

CREATE OR REPLACE TYPE tp_estabelecimento AS OBJECT(
CodEstabelecimento NUMBER(5,0),
Nome VARCHAR2(80),
Tipo VARCHAR2(15),
Endereco tp_endereco,
Telefone tp_telefones
);
/

CREATE TABLE tb_estabelecimento OF tp_estabelecimento(
CodEstabelecimento PRIMARY KEY
);

CREATE OR REPLACE TYPE tp_fatura AS OBJECT(
IdFatura NUMBER(10,0),
ref_titular REF tp_titular,
DataVencimento DATE,
DataPagamento DATE,
ValorPago NUMBER(6,2)
);
/

CREATE TABLE tb_fatura OF tp_fatura(
IdFatura PRIMARY KEY,
ref_titular SCOPE IS tb_titular
);

CREATE OR REPLACE TYPE tp_atende AS OBJECT(
IdAtendimento NUMBER(6,0),
ref_cliente REF tp_cliente,
ref_estabelecimento REF tp_estabelecimento,
DataAtendimento DATE,
ValorDespesa NUMBER(6,2)
);
/

CREATE TABLE tb_atende OF tp_atende(
IdAtendimento PRIMARY KEY,
ref_cliente SCOPE IS tb_cliente,
ref_estabelecimento SCOPE IS tb_estabelecimento
);

CREATE OR REPLACE TYPE tp_possui AS OBJECT(
ref_medico REF tp_medico,
ref_estabelecimento REF tp_estabelecimento
);
/

CREATE TABLE tb_possui OF tp_possui(
ref_medico WITH ROWID REFERENCES tb_medico,
ref_estabelecimento WITH ROWID REFERENCES tb_estabelecimento
);

